/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;

/**
 * TIC3 Netzgebietbeschreibung.
 * 
 * <p>
 * TIC Info XML Element: Description (DES), Descriptive information about the
 * area such as the name and category, normally as a copy of the referenced
 * "Area" data.
 * </p>
 * <p>
 * Entspricht TIC3 NetworkAreaDescriptionType.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */
public class TIC3NetzGebietBeschreibung extends AbstractTIC3InfoElement {

	/**
	 * TIC Info XML Element: Description (DES), Descriptive information about
	 * the area such as the name and category, normally as a copy of the
	 * referenced "Area" data.
	 */
	public static final String XML_ELEMENT_NETWORKAREA_DESCRIPTION = "Description";

	/**
	 * DAV-Attribut.
	 */
	private static final String DAV_ATTR_NETWORKAREA_DESCRIPTION = "GebietsBeschreibung";

	/**
	 * Umschlie&szlig;endes Gebiet. TIC3 Info XML Element: EnclosingArea (ARE),
	 * A list of enclosing areas.
	 */
	private List<TIC3NetzUmschliessendesGebiet> netzUmschliessendesGebiet = new ArrayList<>();

	private boolean hasData;

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3NetzGebietBeschreibung() {
		super(XML_ELEMENT_NETWORKAREA_DESCRIPTION, DAV_ATTR_NETWORKAREA_DESCRIPTION);

		/**
		 * Name des Gebietes. TIC3 Info XML Element: Name (NAM), The name of the
		 * object, in one or more languages configured on the system.
		 */
		addKindElement(new TIC3Text("Name", "TIC3GebietName"));

		/**
		 * Kategorie des Gebietes. TIC3 Info XML Element: Category (CAT), The
		 * area category such as "Nature", "Administrative area", and "Built-up
		 * area".
		 */
		addKindElement(new TIC3Text("Category", "TIC3GebietKategorie", "value"));

		/**
		 * Unterkategorie des Gebietes. TIC3 Info XML Element: SubCategory
		 * (SCA), The area subcategory such as "Country", "Sea", or "Forest".
		 */
		addKindElement(new TIC3Text("SubCategory", "TIC3GebietUnterkategorie", "value"));

		/**
		 * Land des Gebietes. TIC3 Info XML Element: Country (CTR), A reference
		 * to the "Country" which contains the location.
		 */
		addKindElement(new TIC3Text("Country", "TIC3GebietLand", "value"));
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		hasData = true;
		super.einlesenXML(node);
		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);
			if (n.getNodeName().equals(TIC3NetzUmschliessendesGebiet.XML_ELEMENT_NETWORKAREA_ENCLOSING)) {
				final TIC3NetzUmschliessendesGebiet loc = new TIC3NetzUmschliessendesGebiet();
				loc.einlesenXML(n);
				netzUmschliessendesGebiet.add(loc);
			}
		}
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		if (hasData) {
			super.publizieren(davdata, parameter);

			final Data ddata = getLocalDavData(davdata);

			if (netzUmschliessendesGebiet.size() > 0) {
				final Data.Array array = ddata.getArray(TIC3NetzUmschliessendesGebiet.DAV_ATTR_NETWORKAREA_ENCLOSING);
				array.setLength(netzUmschliessendesGebiet.size());
				for (int i = 0; i < netzUmschliessendesGebiet.size(); i++) {
					netzUmschliessendesGebiet.get(i).publizieren(array.getItem(i), parameter);
				}
			}
		}
	}
}
