/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Boolean;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3OptionalerDoubleWert;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TMCInteger;

/**
 * TIC3-Lokationsinformationen auf der Basis von TMC-Codes.
 * 
 * <p>
 * Entspricht TIC3 Info Typ 'TmcLocationType'.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */
public class TIC3TMCLocation extends AbstractTIC3InfoElement {

	/**
	 * TIC3 Info XML Element: TmcLocation (TMC), A list of information about TMC
	 * coded locations.
	 */
	public static final String XML_ELEMENT_TMCLOC = "TmcLocation";

	/**
	 * DAV-Attribut.
	 */
	public static final String DAV_ATTR_TMCLOC = "TIC3TmcLocation";

	private static final String XML_ELEMENT_LOCATIONDIRECTION = "Direction";
	private static final String XML_ELEMENT_LOCATIONAUSMASS = "Extent";

	/**
	 * TIC3 Info XML Element: PrimaryLocation (PRI), The TMC location code of
	 * the primary location.
	 */
	private static final String XML_ELEMENT_LOCATIONPRIMAER = "PrimaryLocation";
	private final TMCInteger primaerLokation = new TMCInteger(XML_ELEMENT_LOCATIONPRIMAER, "PrimaerLokation");

	/**
	 * TIC3 Info XML Element: SecondaryLocation (SEC), The TMC location code of
	 * the secondary location.
	 */
	private static final String XML_ELEMENT_LOCATIONSEKUNDAER = "SecondaryLocation";
	private final TMCInteger sekundaerLokation = new TMCInteger(XML_ELEMENT_LOCATIONSEKUNDAER, "SekundaerLokation");

	/**
	 * TIC3 Info XML Element: DistancePrimary (DPR), The distance between the
	 * end of the incident and the primary location.
	 */
	private static final String XML_ELEMENT_LOCATIONABSTANDPRIMAER = "DistancePrimaryLocation";
	private final TIC3OptionalerDoubleWert primaerEntfernung = new TIC3OptionalerDoubleWert(
			XML_ELEMENT_LOCATIONABSTANDPRIMAER, "PrimaerEntfernung");

	/**
	 * TIC3 Info XML Element: DistanceSecondaryLocation (DSE), he distance
	 * between the secondary location and the begin of the incident.
	 */
	private static final String XML_ELEMENT_LOCATIONABSTANDSEKUNDAER = "DistanceSecondaryLocation";
	private final TIC3OptionalerDoubleWert sekundaerEntfernung = new TIC3OptionalerDoubleWert(
			XML_ELEMENT_LOCATIONABSTANDSEKUNDAER, "SekundaerEntfernung");

	private static final String XML_ELEMENT_LOCATIONLAENGE = "LocationLength";

	private static final String XML_ELEMENT_LOCATIONPRIMAEREINAUSFAHRT = "PrimaryLocationCodeIsUsedForEntranceOrExit";

	/**
	 * TIC Info XML Element: Direction (DIR), The direction indicates the TMC
	 * direction of queue growth. It can be either 0 (positive) or 1 (negative).
	 * <p>
	 * Default: 'ohne Richtung': -1
	 * </p>
	 */
	private final TMCInteger tmcRichtung = new TMCInteger(XML_ELEMENT_LOCATIONDIRECTION, "TMCRichtung", -1);

	/**
	 * Erweiterter Ländercode der Locationtabelle.
	 * <p>
	 * (Wird hier als Text modelliert, da uns aktuell die IEC 62106 nicht
	 * vorliegt)
	 * </p>
	 */
	private TIC3Text locationTabelleErweiterterLandescode = new TIC3Text("LocationTableExtendedCountryCode",
			"LocationTabelleErweiterterLandescode");

	/**
	 * Ländercode der Locationtabelle.
	 * <p>
	 * (Wird hier als Text modelliert, da uns aktuell die IEC 62106 nicht
	 * vorliegt)
	 * </p>
	 */
	private TIC3Text locationTabelleLandescode = new TIC3Text("LocationTableCountryCode", "LocationTabelleLandescode");

	/**
	 * Nummer der Locationtabelle.
	 */
	private TIC3Text locationTabelleNummer = new TIC3Text("LocationTableNumber", "LocationTabelleNummer");

	/**
	 * Version der Locationtabelle.
	 */
	private TIC3Text locationTabelleVersion = new TIC3Text("LocationTableVersion", "LocationTabelleVersion");

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3TMCLocation() {
		super(XML_ELEMENT_TMCLOC, DAV_ATTR_TMCLOC);
		addKindElement(locationTabelleErweiterterLandescode);
		addKindElement(locationTabelleLandescode);
		addKindElement(locationTabelleNummer);
		addKindElement(locationTabelleVersion);
		addKindElement(tmcRichtung);
		addKindElement(new TMCInteger(XML_ELEMENT_LOCATIONAUSMASS, "Ausmass"));
		addKindElement(primaerLokation);
		addKindElement(sekundaerLokation);
		addKindElement(primaerEntfernung);
		addKindElement(sekundaerEntfernung);
		addKindElement(new TIC3OptionalerDoubleWert(XML_ELEMENT_LOCATIONLAENGE, "Laenge"));
		addKindElement(new TIC3Boolean(XML_ELEMENT_LOCATIONPRIMAEREINAUSFAHRT, "PrimaerLokationIstEinAusfahrt"));
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		// Locationtable-Informationen werden zusammengefasst
		final Data locationTableData = davdata.getItem("LocationTabelleInfo");

		for (final TIC3InfoElement child : getKindElemente()) {
			if (child.getXmlElementName().startsWith("LocationTable")) {
				child.publizieren(locationTableData, parameter);
			} else {
				child.publizieren(davdata, parameter);
			}
		}
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName();
	}

	public Integer getPrimaerLokation() {
		return primaerLokation.getWert();
	}

	public void setPrimaerLokation(final Integer loc) {
		primaerLokation.setWert(loc);
	}

	public Double getPrimaerEntfernung() {
		return primaerEntfernung.getWert();
	}

	public void setPrimaerEntfernung(final Double wert) {
		primaerEntfernung.setWert(wert);
	}

	public Integer getSekundaerLokation() {
		return sekundaerLokation.getWert();
	}

	public void setSekundaerLokation(final Integer loc) {
		sekundaerLokation.setWert(loc);
	}

	public Double getSekundaerEntfernung() {
		return sekundaerEntfernung.getWert();
	}

	public void setSekundaerEntfernung(final Double wert) {
		sekundaerEntfernung.setWert(wert);
	}

	public Integer getTmcRichtung() {
		return tmcRichtung.getWert();
	}
}
