/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Boolean;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoAttribut;

/**
 * TIC3-Zeitplan.
 * <p>
 * Entspricht TIC3-Typ 'ScheduleType'.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */
public class TIC3Zeitplan extends AbstractTIC3InfoElement {

	/**
	 * Beschreibung.
	 * <p>
	 * Beschreibung aus Wert des 'description'-Attributes.
	 * </p>
	 */
	private TIC3InfoAttribut beschreibung = new TIC3InfoAttribut("description", "Beschreibung");

	/**
	 * Startzeit.
	 * <p>
	 * TIC3 Info XML Element: StartTime (BGN), The date and time when the
	 * overall schedule starts.
	 * </p>
	 */
	private final TIC3Zeitstempel startzeit = new TIC3Zeitstempel("StartTime", "StartZeit");

	/**
	 * StartZeitUnbekannt.
	 * <p>
	 * TIC3 Info XML Element: StartTimeUnknown (STU), Specifies whether or not
	 * the start time is unknown, and when unknown, the start time is assumed to
	 * be at midnight.
	 * </p>
	 */
	private final TIC3Boolean startzeitUnbekannt = new TIC3Boolean("StartTimeUnknown", "StartZeitUnbekannt");

	/**
	 * StartetUmMitternacht.
	 * <p>
	 * TIC3 Info XML Element: StartsAtMidnight (SAM), Specifies whether or not
	 * the start time is at midnight.
	 * </p>
	 */
	private final TIC3Boolean startetUmMitternacht = new TIC3Boolean("StartsAtMidnight", "StartetUmMitternacht");

	/**
	 * Startzeit ungefähr..
	 * <p>
	 * TIC3 Info XML Element: StartTimeApproximate (STP), Specifies whether or
	 * not the start time is approximate to indicate that the event will start
	 * shortly before or after.
	 * </p>
	 */
	private final TIC3Boolean startzeitUngefaehr = new TIC3Boolean("StartTimeApproximate", "StartzeitUngefähr");

	/**
	 * EndZeit.
	 * <p>
	 * TIC3 Info XML Element: TIC3 Info XML Element: EndTime (END), The date and
	 * time when the overall schedule ends.
	 * </p>
	 */
	private final TIC3Zeitstempel endZeit = new TIC3Zeitstempel("EndTime", "EndZeit");

	/**
	 * EndZeitUnbekannt.
	 * <p>
	 * TIC3 Info XML Element: EndTimeUnknown (ETU), Specifies whether or not the
	 * end time is unknown, and when unknown, the end time is assumed to be just
	 * before midnight.
	 * </p>
	 */
	private final TIC3Boolean endZeitUnbekannt = new TIC3Boolean("EndTimeUnknown", "EndZeitUnbekannt");

	/**
	 * EndetUmMitternacht.
	 * <p>
	 * TIC3 Info XML Element: EndsAtMidnight (EAM), Specifies whether or not the
	 * end time is just before midnight.
	 * </p>
	 */
	private final TIC3Boolean endetUmMitternacht = new TIC3Boolean("EndsAtMidnight", "EndetUmMitternacht");

	/**
	 * Endzeit ungefähr.
	 * <p>
	 * TIC3 Info XML Element: EndTimeApproximate (ETP), Specifies whether or not
	 * the end time is approximate to indicate that the event will end shortly
	 * before or after.
	 * </p>
	 */
	private final TIC3Boolean endzeitUngefaehr = new TIC3Boolean("EndTimeApproximate", "EndzeitUngefähr");

	/**
	 * Wiederholung.
	 * <p>
	 * TIC3 Info XML Element: Recurrence (REC), A list of recurring periods
	 * between the overall begin and end times during which the overall schedule
	 * is limited to specific days and times.
	 * </p>
	 */
	// new TIC3Wiederholung("Recurrence", "Wiederholung");
	private final List<TIC3Wiederholung> wiederholungen = new ArrayList<>();

	private boolean hasData;

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3Zeitplan(final String xmlName, final String davName) {
		super(xmlName, davName);
		addAttribut(beschreibung);
		addKindElement(startzeit);
		addKindElement(startzeitUnbekannt);
		addKindElement(startzeitUngefaehr);
		addKindElement(startetUmMitternacht);
		addKindElement(endZeit);
		addKindElement(endZeitUnbekannt);
		addKindElement(endzeitUngefaehr);
		addKindElement(endetUmMitternacht);
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		super.einlesenXML(node);

		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);
			if (n.getNodeName().equals(TIC3Wiederholung.XML_ELEMENT_WIEDERHOLUNG)) {
				final TIC3Wiederholung loc = new TIC3Wiederholung(TIC3Wiederholung.XML_ELEMENT_WIEDERHOLUNG,
						TIC3Wiederholung.DAV_ATTR_WIEDERHOLUNG);
				loc.einlesenXML(n);
				wiederholungen.add(loc);
			}
		}
		hasData = true;
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		if (hasData) {
			super.publizieren(davdata, parameter);
		}
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) throws TIC3Exception {
		// das wird nicht exportiert
		return null;
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName() + ": Beschreibung='" + beschreibung.getXmlValue() + "'";
	}
}
